package Sys::Info::Driver;
$Sys::Info::Driver::VERSION = '0.7809';
use strict;
use warnings;

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Sys::Info::Driver

=head1 VERSION

version 0.7809

=head1 NAME

Sys::Info::Driver - Driver base class

=head1 SEE ALSO

L<Sys::Info>,
L<Sys::Info::Driver::Unknown>,
L<Sys::Info::Driver::Windows>,
L<Sys::Info::Driver::Linux>,
L<Sys::Info::Driver::BSD>.

=head1 AUTHOR

Burak Gursoy

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2006 by Burak Gursoy.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
